@@ -6,25 +6,15 @@ import sys
 import traceback
 from importlib import import_module
 from types import MethodType
-from typing import TYPE_CHECKING, Any, Callable, Iterator
-
-from docutils import nodes
-from docutils.core import Publisher
-from docutils.nodes import Element, Node, TextElement
-from docutils.parsers import Parser
-from docutils.parsers.rst import Directive
-from docutils.transforms import Transform
+from typing import TYPE_CHECKING, Any, Callable
 
 if sys.version_info >= (3, 10):
     from importlib.metadata import entry_points
 else:
     from importlib_metadata import entry_points
 
-from sphinx.builders import Builder
-from sphinx.config import Config
 from sphinx.domains import Domain, Index, ObjType
 from sphinx.domains.std import GenericObject, Target
-from sphinx.environment import BuildEnvironment
 from sphinx.errors import ExtensionError, SphinxError, VersionRequirementError
 from sphinx.extension import Extension
 from sphinx.io import create_publisher
@@ -33,11 +23,28 @@ from sphinx.parsers import Parser as SphinxParser
 from sphinx.roles import XRefRole
 from sphinx.util import logging
 from sphinx.util.logging import prefixed_warnings
-from sphinx.util.typing import RoleFunction, TitleGetter
 
 if TYPE_CHECKING:
+    from collections.abc import Iterator, Sequence
+
+    from docutils import nodes
+    from docutils.core import Publisher
+    from docutils.nodes import Element, Node, TextElement
+    from docutils.parsers import Parser
+    from docutils.parsers.rst import Directive
+    from docutils.transforms import Transform
+
     from sphinx.application import Sphinx
+    from sphinx.builders import Builder
+    from sphinx.config import Config
+    from sphinx.environment import BuildEnvironment
     from sphinx.ext.autodoc import Documenter
+    from sphinx.util.typing import (
+        ExtensionMetadata,
+        RoleFunction,
+        TitleGetter,
+        _ExtensionSetupFunc,
+    )
 
 logger = logging.getLogger(__name__)
 
@@ -45,6 +52,7 @@ logger = logging.getLogger(__name__)
 # Values are Sphinx version that merge the extension.
 EXTENSION_BLACKLIST = {
     "sphinxjp.themecore": "1.2",
+    "sphinxprettysearchresults": "2.0.0",
 }
 
 
@@ -53,7 +61,7 @@ class SphinxComponentRegistry:
         #: special attrgetter for autodoc; class object -> attrgetter
         self.autodoc_attrgettrs: dict[type, Callable[[Any, str, Any], Any]] = {}
 
-        #: builders; a dict of builder name -> bulider class
+        #: builders; a dict of builder name -> builder class
         self.builders: dict[str, type[Builder]] = {}
 
         #: autodoc documenters; a dict of documenter name -> documenter class
@@ -83,12 +91,14 @@ class SphinxComponentRegistry:
 
         #: additional enumerable nodes
         #: a dict of node class -> tuple of figtype and title_getter function
-        self.enumerable_nodes: dict[type[Node], tuple[str, TitleGetter]] = {}
+        self.enumerable_nodes: dict[type[Node], tuple[str, TitleGetter | None]] = {}
 
         #: HTML inline and block math renderers
         #: a dict of name -> tuple of visit function and depart function
-        self.html_inline_math_renderers: dict[str, tuple[Callable, Callable]] = {}
-        self.html_block_math_renderers: dict[str, tuple[Callable, Callable]] = {}
+        self.html_inline_math_renderers: dict[str,
+                                              tuple[Callable, Callable | None]] = {}
+        self.html_block_math_renderers: dict[str,
+                                             tuple[Callable, Callable | None]] = {}
 
         #: HTML assets
         self.html_assets_policy: str = 'per_page'
@@ -97,12 +107,12 @@ class SphinxComponentRegistry:
         self.html_themes: dict[str, str] = {}
 
         #: js_files; list of JS paths or URLs
-        self.js_files: list[tuple[str, dict[str, Any]]] = []
+        self.js_files: list[tuple[str | None, dict[str, Any]]] = []
 
         #: LaTeX packages; list of package names and its options
-        self.latex_packages: list[tuple[str, str]] = []
+        self.latex_packages: list[tuple[str, str | None]] = []
 
-        self.latex_packages_after_hyperref: list[tuple[str, str]] = []
+        self.latex_packages_after_hyperref: list[tuple[str, str | None]] = []
 
         #: post transforms; list of transforms
         self.post_transforms: list[type[Transform]] = []
@@ -118,7 +128,7 @@ class SphinxComponentRegistry:
 
         #: custom handlers for translators
         #: a dict of builder name -> dict of node name -> visitor and departure functions
-        self.translation_handlers: dict[str, dict[str, tuple[Callable, Callable]]] = {}
+        self.translation_handlers: dict[str, dict[str, tuple[Callable, Callable | None]]] = {}
 
         #: additional transforms; list of transforms
         self.transforms: list[type[Transform]] = []
@@ -149,8 +159,7 @@ class SphinxComponentRegistry:
 
             self.load_extension(app, entry_point.module)
 
-    def create_builder(self, app: Sphinx, name: str,
-                       env: BuildEnvironment | None = None) -> Builder:
+    def create_builder(self, app: Sphinx, name: str, env: BuildEnvironment) -> Builder:
         if name not in self.builders:
             raise SphinxError(__('Builder name %s not registered') % name)
 
@@ -221,7 +230,7 @@ class SphinxComponentRegistry:
         parse_node: Callable | None = None,
         ref_nodeclass: type[TextElement] | None = None,
         objname: str = '',
-        doc_field_types: list = [],
+        doc_field_types: Sequence = (),
         override: bool = False,
     ) -> None:
         logger.debug('[app] adding object type: %r',
@@ -232,7 +241,7 @@ class SphinxComponentRegistry:
         directive = type(directivename,
                          (GenericObject, object),
                          {'indextemplate': indextemplate,
-                          'parse_node': staticmethod(parse_node),
+                          'parse_node': parse_node and staticmethod(parse_node),
                           'doc_field_types': doc_field_types})
 
         self.add_directive_to_domain('std', directivename, directive)
@@ -309,8 +318,11 @@ class SphinxComponentRegistry:
             raise ExtensionError(__('Translator for %r already exists') % name)
         self.translators[name] = translator
 
-    def add_translation_handlers(self, node: type[Element],
-                                 **kwargs: tuple[Callable, Callable]) -> None:
+    def add_translation_handlers(
+        self,
+        node: type[Element],
+        **kwargs: tuple[Callable, Callable | None],
+    ) -> None:
         logger.debug('[app] adding translation_handlers: %r, %r', node, kwargs)
         for builder_name, handlers in kwargs.items():
             translation_handlers = self.translation_handlers.setdefault(builder_name, {})
@@ -324,12 +336,17 @@ class SphinxComponentRegistry:
                 ) from exc
 
     def get_translator_class(self, builder: Builder) -> type[nodes.NodeVisitor]:
-        return self.translators.get(builder.name,
-                                    builder.default_translator_class)
+        try:
+            return self.translators[builder.name]
+        except KeyError:
+            try:
+                return builder.default_translator_class
+            except AttributeError as err:
+                msg = f'translator not found for {builder.name}'
+                raise AttributeError(msg) from err
 
     def create_translator(self, builder: Builder, *args: Any) -> nodes.NodeVisitor:
         translator_class = self.get_translator_class(builder)
-        assert translator_class, "translator not found for %s" % builder.name
         translator = translator_class(*args)
 
         # transplant handlers for custom nodes to translator instance
@@ -369,7 +386,7 @@ class SphinxComponentRegistry:
     def add_css_files(self, filename: str, **attributes: Any) -> None:
         self.css_files.append((filename, attributes))
 
-    def add_js_file(self, filename: str, **attributes: Any) -> None:
+    def add_js_file(self, filename: str | None, **attributes: Any) -> None:
         logger.debug('[app] adding js_file: %r, %r', filename, attributes)
         self.js_files.append((filename, attributes))
 
@@ -377,7 +394,9 @@ class SphinxComponentRegistry:
         packages = self.latex_packages + self.latex_packages_after_hyperref
         return bool([x for x in packages if x[0] == name])
 
-    def add_latex_package(self, name: str, options: str, after_hyperref: bool = False) -> None:
+    def add_latex_package(
+        self, name: str, options: str | None, after_hyperref: bool = False,
+    ) -> None:
         if self.has_latex_package(name):
             logger.warning("latex package '%s' already included", name)
 
@@ -398,16 +417,21 @@ class SphinxComponentRegistry:
             raise ExtensionError(__('enumerable_node %r already registered') % node)
         self.enumerable_nodes[node] = (figtype, title_getter)
 
-    def add_html_math_renderer(self, name: str,
-                               inline_renderers: tuple[Callable, Callable],
-                               block_renderers: tuple[Callable, Callable]) -> None:
+    def add_html_math_renderer(
+        self,
+        name: str,
+        inline_renderers: tuple[Callable, Callable | None] | None,
+        block_renderers: tuple[Callable, Callable | None] | None,
+    ) -> None:
         logger.debug('[app] adding html_math_renderer: %s, %r, %r',
                      name, inline_renderers, block_renderers)
         if name in self.html_inline_math_renderers:
             raise ExtensionError(__('math renderer %s is already registered') % name)
 
-        self.html_inline_math_renderers[name] = inline_renderers
-        self.html_block_math_renderers[name] = block_renderers
+        if inline_renderers is not None:
+            self.html_inline_math_renderers[name] = inline_renderers
+        if block_renderers is not None:
+            self.html_block_math_renderers[name] = block_renderers
 
     def add_html_theme(self, name: str, theme_path: str) -> None:
         self.html_themes[name] = theme_path
@@ -432,11 +456,11 @@ class SphinxComponentRegistry:
                 raise ExtensionError(__('Could not import extension %s') % extname,
                                      err) from err
 
-            setup = getattr(mod, 'setup', None)
+            setup: _ExtensionSetupFunc | None = getattr(mod, 'setup', None)
             if setup is None:
                 logger.warning(__('extension %r has no setup() function; is it really '
                                   'a Sphinx extension module?'), extname)
-                metadata: dict[str, Any] = {}
+                metadata: ExtensionMetadata = {}
             else:
                 try:
                     metadata = setup(app)
@@ -458,7 +482,7 @@ class SphinxComponentRegistry:
 
             app.extensions[extname] = Extension(extname, mod, **metadata)
 
-    def get_envversion(self, app: Sphinx) -> dict[str, str]:
+    def get_envversion(self, app: Sphinx) -> dict[str, int]:
         from sphinx.environment import ENV_VERSION
         envversion = {ext.name: ext.metadata['env_version'] for ext in app.extensions.values()
                       if ext.metadata.get('env_version')}
@@ -489,7 +513,7 @@ def merge_source_suffix(app: Sphinx, config: Config) -> None:
     app.registry.source_suffix = app.config.source_suffix
 
 
-def setup(app: Sphinx) -> dict[str, Any]:
+def setup(app: Sphinx) -> ExtensionMetadata:
     app.connect('config-inited', merge_source_suffix, priority=800)
 
     return {
