@@ -1,14 +1,8 @@
-"""
-    sphinx.search.tr
-    ~~~~~~~~~~~~~~~~
+"""Turkish search language: includes the JS Turkish stemmer."""
 
-    Turkish search language: includes the JS Turkish stemmer.
+from __future__ import annotations
 
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-from typing import Dict, Set
+from typing import TYPE_CHECKING, Dict, Set
 
 import snowballstemmer
 
@@ -19,9 +13,9 @@ class SearchTurkish(SearchLanguage):
     lang = 'tr'
     language_name = 'Turkish'
     js_stemmer_rawcode = 'turkish-stemmer.js'
-    stopwords = set()  # type: Set[str]
+    stopwords: set[str] = set()
 
-    def init(self, options: Dict) -> None:
+    def init(self, options: dict[str, str]) -> None:
         self.stemmer = snowballstemmer.stemmer('turkish')
 
     def stem(self, word: str) -> str:
